/*
** patcher.c
** 31st Dec 2006 
**
** Version 0.2 - Code tidy up and prep for source release.  
**             - Code relocation aswell, removes chance of overwriting code in tightly packed files
**             - [BUG FIXED] Not copying the correct size to file?
**             - Check for data where we would be injectecting. If found Warn and abandon (add FORCE option?)
**
*/

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[]) {

  int moonboot = 0x2380000;

  FILE *fp;
  FILE *fp1;
  unsigned int offset;
  unsigned int datasize;
  unsigned int entryPoint;
  unsigned int RAMaddress;
  unsigned char *bdata;
  unsigned char *patchdata;
  int loadersize;
  int patchsize;
  int count = 0;
  int i;
  unsigned char tmp;



  if(argc!=2) {
    printf("%s X9 Homebrew patch tool version 0.2 - Bobmcbob+blasty\n",argv[0]);
    printf("Make sure you have patch.bin in the same folder. Then ./%s <ROM.nds>",argv[0]);
    exit(0);
  }
  
  printf("%s X9 Homebrew patch tool version 0.2 - Bobmcbob+blasty\n",argv[0]);

  //Read data from rom header

  if((fp = fopen(argv[1],"rb"))==NULL) {
    printf("Rom file not found \n");
    exit(0);
  }
     
  fseek(fp,0x30, SEEK_SET);
  fread(&offset,1,sizeof(unsigned int),fp);
  fseek(fp,0x34, SEEK_SET);
  fread(&entryPoint,1,sizeof(unsigned int),fp);
  fseek(fp,0x38, SEEK_SET);
  fread(&RAMaddress,1,sizeof(unsigned int),fp);
  fseek(fp,0x3C, SEEK_SET);
  fread(&datasize,1,sizeof(unsigned int),fp);

  printf("ARM7 ROM Offset: 0x%x \n", offset);
  printf("ARM7 RAM Offset: 0x%x \n", RAMaddress);
  printf("ARM7 ENTRY POINT: 0x%x \n", entryPoint);
  printf("ARM7 DataSize: 0x%x \n", datasize);
  
  //Allocate mem for existing arm7 binary
  
  bdata = (unsigned char*)malloc(datasize);

  //Copy arm7 binary to bdata

  fseek(fp,offset,SEEK_SET);
  for (count=0;count!=datasize;count++) {
    fread(&bdata[count],1,1,fp);
  }
 
  fclose(fp);

  if((fp = fopen("patch.bin","r+b"))==NULL) {
    printf("patch.bin not found\n");
    exit(0);
  }
  //get loader size (original patch)
  fseek(fp,0x0,SEEK_END);
  loadersize=(ftell(fp))-4;
  printf ("Loader code size = %x \n",loadersize);
  //Seek to end of loader code (0xDDDDDD) in the patch file and write arm7 to it
  //for debug really, no real use
  fseek(fp, loadersize, SEEK_SET);
  for(count=0;count!=datasize;count++) {
    fwrite(&bdata[count],1,1,fp); 
  }
  
  //Patch in the proper entry points
  fseek(fp, 0x2c, SEEK_SET);
  fwrite(&RAMaddress,1,sizeof(unsigned int),fp);
  fseek(fp, 0x30, SEEK_SET);
  fwrite(&datasize,1,sizeof(unsigned int),fp);
  fseek(fp, 0x34, SEEK_SET);
  fwrite(&RAMaddress,1,sizeof(unsigned int),fp);
  
  //get the final size of the patch with the code added
  fseek(fp,0x0, SEEK_END);
  patchsize = ftell(fp);

  //reset the file pointer so we at the begining
  rewind(fp);

  fp1 = fopen(argv[1],"r+b");

  printf("Patching ROM entrypoints..\n");
  fseek(fp1, 0x34, SEEK_SET);
  fwrite(&moonboot,1,sizeof(unsigned int),fp1);
  fseek(fp1, 0x38, SEEK_SET);
  fwrite(&moonboot,1,sizeof(unsigned int),fp1);


  //Check for enough room AFTER the orig ARM7 to add our loader
  //warn and quit if not. Should really move the arm7 outside o
  //of the normal rom but cant be arsed.

  fseek(fp1, offset+datasize, SEEK_SET);

  for(i=1;i!=loadersize;i++){
    fread(&tmp,1,1,fp1);
    if(tmp!=0x0) {
      printf("[!]Warning:  Patching ths could possibly overflow into data areas...quiting\n");
      exit(0);
    }
  }

  printf("Adding bootloader..\n");
  //Seek to where the orig arm7 is so we can overwrite
  fseek(fp1, offset, SEEK_SET);
  //read and write patched data
  for(count=0;count!=patchsize;count++) {
    fread(&tmp,1,1,fp);
    fwrite(&tmp,1,1,fp1);
  }

  fclose(fp1);
  fclose(fp);

}





